﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2008 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.ServiceModel;
using System.ServiceModel.Description;
using System.Windows.Forms;
using HeuristicLab.Hive.Contracts;
using HeuristicLab.Hive.Contracts.Interfaces;
using HeuristicLab.PluginInfrastructure;
using HeuristicLab.Hive.Server.Core;

namespace HeuristicLab.Hive.Server {
  [Application("Hive Server", "Server application for the distributed hive engine.", true)]
  public class HeuristicLabHiveServerApplication : ApplicationBase {
    public const string STR_SlaveCommunicator = "HiveServer";
    public const string STR_ServerConsoleFacade = "ServerConsoleFacade";
    public const string STR_ExecutionEngineFacade = "ExecutionEngineFacade";
    Dictionary<string, Uri> baseAddrDict = new Dictionary<string, Uri>();

    public override void Run() {
      ServiceHost slaveHost = new ServiceHost(ApplicationManager.Manager.GetTypes(typeof(ISlaveFacade)).First(), GetUri(STR_SlaveCommunicator));
      slaveHost.Open();

      ServiceHost serverConsoleHost = new ServiceHost(ApplicationManager.Manager.GetTypes(typeof(IServerConsoleFacade)).First(), GetUri(STR_ServerConsoleFacade));
      serverConsoleHost.Open();

      ServiceHost executionEngineHost = new ServiceHost(ApplicationManager.Manager.GetTypes(typeof(IExecutionEngineFacade)).First(), GetUri(STR_ExecutionEngineFacade));
      executionEngineHost.Open();

      ILifecycleManager lifecycleManager = ServiceLocator.GetLifecycleManager();
      lifecycleManager.Init();

      Form mainForm = new MainForm(baseAddrDict);
      Application.Run();

      slaveHost.Close();
      serverConsoleHost.Close();
      executionEngineHost.Close();

      lifecycleManager.Shutdown();
    }

    private Uri GetUri(string serviceName) {
      string ipAddress = WcfSettings.GetActiveIP().ToString();
      string port = WcfSettings.DEFAULTPORT.ToString();
      Uri uriHttp = new Uri("http://" + ipAddress + ":" + port + "/" + serviceName + "/");
      baseAddrDict.Add(serviceName, uriHttp);
      return uriHttp;
    }

  }
}
